@@ -42,6 +42,7 @@
 #include <kio/copyjob.h>
 #include <kio/deletejob.h>
 #include <kio/chmodjob.h>
+#include <kio/statjob.h>
 #include "kiotesthelper.h" // createTestFile etc.
 #ifndef Q_OS_WIN
 #include <unistd.h> // for readlink
@@ -114,7 +115,7 @@ void JobTest::enterLoop()
 
 void JobTest::storedGet()
 {
-    qDebug();
+    // qDebug();
     const QString filePath = homeTmpDir() + "fileFromHome";
     createTestFile(filePath);
     QUrl u = QUrl::fromLocalFile(filePath);
@@ -599,7 +600,7 @@ void JobTest::copyFileToSamePartition()
 
 void JobTest::copyDirectoryToSamePartition()
 {
-    qDebug();
+    // qDebug();
     const QString src = homeTmpDir() + "dirFromHome";
     const QString dest = homeTmpDir() + "dirFromHome_copied";
     createTestDirectory(src);
@@ -608,7 +609,7 @@ void JobTest::copyDirectoryToSamePartition()
 
 void JobTest::copyDirectoryToExistingDirectory()
 {
-    qDebug();
+    // qDebug();
     // just the same as copyDirectoryToSamePartition, but this time dest exists.
     // So we get a subdir, "dirFromHome_copy/dirFromHome"
     const QString src = homeTmpDir() + "dirFromHome";
@@ -618,9 +619,41 @@ void JobTest::copyDirectoryToExistingDirectory()
     copyLocalDirectory(src, dest, AlreadyExists);
 }
 
+void JobTest::copyDirectoryToExistingSymlinkedDirectory()
+{
+    // qDebug();
+    // just the same as copyDirectoryToSamePartition, but this time dest is a symlink.
+    // So we get a file in the symlink dir, "dirFromHome_symlink/dirFromHome" and
+    // "dirFromHome_symOrigin/dirFromHome"
+    const QString src = homeTmpDir() + "dirFromHome";
+    const QString origSymlink = homeTmpDir() + "dirFromHome_symOrigin";
+    const QString targetSymlink = homeTmpDir() + "dirFromHome_symlink";
+    createTestDirectory(src);
+    createTestDirectory(origSymlink);
+
+    bool ok = KIOPrivate::createSymlink(origSymlink, targetSymlink);
+    if (!ok) {
+        qFatal("couldn't create symlink: %s", strerror(errno));
+    }
+    QVERIFY(QFileInfo(targetSymlink).isSymLink());
+    QVERIFY(QFileInfo(targetSymlink).isDir());
+
+    KIO::Job *job = KIO::copy(QUrl::fromLocalFile(src), QUrl::fromLocalFile(targetSymlink), KIO::HideProgressInfo);
+    job->setUiDelegate(nullptr);
+    job->setUiDelegateExtension(nullptr);
+    QVERIFY2(job->exec(), qPrintable(job->errorString()));
+    QVERIFY(QFile::exists(src));     // still there
+
+    // file is visible in both places due to symlink
+    QVERIFY(QFileInfo(origSymlink + "/dirFromHome").isDir());;
+    QVERIFY(QFileInfo(targetSymlink + "/dirFromHome").isDir());
+    QVERIFY(QDir(origSymlink).removeRecursively());
+    QVERIFY(QFile::remove(targetSymlink));
+}
+
 void JobTest::copyFileToOtherPartition()
 {
-    qDebug();
+    // qDebug();
     const QString filePath = homeTmpDir() + "fileFromHome";
     const QString dest = otherTmpDir() + "fileFromHome_copied";
     createTestFile(filePath);
@@ -629,7 +662,7 @@ void JobTest::copyFileToOtherPartition()
 
 void JobTest::copyDirectoryToOtherPartition()
 {
-    qDebug();
+    // qDebug();
     const QString src = homeTmpDir() + "dirFromHome";
     const QString dest = otherTmpDir() + "dirFromHome_copied";
     createTestDirectory(src);
@@ -676,6 +709,7 @@ void JobTest::copyFolderWithUnaccessibleSubfolder()
     createTestDirectory(src_dir);
     createTestDirectory(src_dir + "/folder1");
     QString inaccessible = src_dir + "/folder1/inaccessible";
+
     createTestDirectory(inaccessible);
 
     QFile(inaccessible).setPermissions(QFile::Permissions()); // Make it inaccessible
@@ -1381,8 +1415,21 @@ void JobTest::stat()
     KIO::StatJob *job = KIO::stat(url, KIO::HideProgressInfo);
     QVERIFY(job);
     QVERIFY2(job->exec(), qPrintable(job->errorString()));
-    // TODO set setSide, setDetails
+    // TODO set setSide
     const KIO::UDSEntry &entry = job->statResult();
+
+    // we only get filename, access, type, size, uid, gid, btime, mtime, atime
+    QVERIFY(entry.contains(KIO::UDSEntry::UDS_NAME));
+    QVERIFY(entry.contains(KIO::UDSEntry::UDS_ACCESS));
+    QVERIFY(entry.contains(KIO::UDSEntry::UDS_SIZE));
+    QVERIFY(entry.contains(KIO::UDSEntry::UDS_FILE_TYPE));
+    QVERIFY(entry.contains(KIO::UDSEntry::UDS_USER));
+    QVERIFY(entry.contains(KIO::UDSEntry::UDS_GROUP));
+    //QVERIFY(entry.contains(KIO::UDSEntry::UDS_CREATION_TIME)); // only true if st_birthtime or statx is used
+    QVERIFY(entry.contains(KIO::UDSEntry::UDS_MODIFICATION_TIME));
+    QVERIFY(entry.contains(KIO::UDSEntry::UDS_ACCESS_TIME));
+    QCOMPARE(entry.count(), 8 + (entry.contains(KIO::UDSEntry::UDS_CREATION_TIME) ? 1 : 0));
+
     QVERIFY(!entry.isDir());
     QVERIFY(!entry.isLink());
     QCOMPARE(entry.stringValue(KIO::UDSEntry::UDS_NAME), QStringLiteral("fileFromHome"));
@@ -1413,6 +1460,123 @@ void JobTest::stat()
 #endif
 }
 
+void JobTest::statDetailsBasic()
+{
+    const QString filePath = homeTmpDir() + "fileFromHome";
+    createTestFile(filePath);
+    const QUrl url(QUrl::fromLocalFile(filePath));
+    KIO::StatJob *job = KIO::statDetails(url, KIO::StatJob::StatSide::SourceSide, KIO::StatBasic,  KIO::HideProgressInfo);
+    QVERIFY(job);
+    QVERIFY2(job->exec(), qPrintable(job->errorString()));
+    // TODO set setSide
+    const KIO::UDSEntry &entry = job->statResult();
+
+    // we only get filename, access, type, size, (no linkdest)
+    QVERIFY(entry.contains(KIO::UDSEntry::UDS_NAME));
+    QVERIFY(entry.contains(KIO::UDSEntry::UDS_ACCESS));
+    QVERIFY(entry.contains(KIO::UDSEntry::UDS_SIZE));
+    QVERIFY(entry.contains(KIO::UDSEntry::UDS_FILE_TYPE));
+    QCOMPARE(entry.count(), 4);
+
+    QVERIFY(!entry.isDir());
+    QVERIFY(!entry.isLink());
+    QVERIFY(entry.numberValue(KIO::UDSEntry::UDS_ACCESS) > 0);
+    QCOMPARE(entry.stringValue(KIO::UDSEntry::UDS_NAME), QStringLiteral("fileFromHome"));
+
+    // Compare what we get via kio_file and what we get when KFileItem stat()s directly
+    // for the requested fields
+    const KFileItem kioItem(entry, url);
+    const KFileItem fileItem(url);
+    QCOMPARE(kioItem.name(), fileItem.name());
+    QCOMPARE(kioItem.url(), fileItem.url());
+    QCOMPARE(kioItem.size(), fileItem.size());
+    QCOMPARE(kioItem.user(), "");
+    QCOMPARE(kioItem.group(), "");
+    QCOMPARE(kioItem.mimetype(), "application/octet-stream");
+    QCOMPARE(kioItem.permissions(), 438);
+    QCOMPARE(kioItem.time(KFileItem::ModificationTime), QDateTime());
+    QCOMPARE(kioItem.time(KFileItem::AccessTime), QDateTime());
+}
+
+void JobTest::statDetailsBasicSetDetails()
+{
+    const QString filePath = homeTmpDir() + "fileFromHome";
+    createTestFile(filePath);
+    const QUrl url(QUrl::fromLocalFile(filePath));
+    KIO::StatJob *job = KIO::stat(url);
+    job->setDetails(KIO::StatBasic);
+    QVERIFY(job);
+    QVERIFY2(job->exec(), qPrintable(job->errorString()));
+    // TODO set setSide
+    const KIO::UDSEntry &entry = job->statResult();
+
+    // we only get filename, access, type, size, (no linkdest)
+    QVERIFY(entry.contains(KIO::UDSEntry::UDS_NAME));
+    QVERIFY(entry.contains(KIO::UDSEntry::UDS_ACCESS));
+    QVERIFY(entry.contains(KIO::UDSEntry::UDS_SIZE));
+    QVERIFY(entry.contains(KIO::UDSEntry::UDS_FILE_TYPE));
+    QCOMPARE(entry.count(), 4);
+
+    QVERIFY(!entry.isDir());
+    QVERIFY(!entry.isLink());
+    QVERIFY(entry.numberValue(KIO::UDSEntry::UDS_ACCESS) > 0);
+    QCOMPARE(entry.stringValue(KIO::UDSEntry::UDS_NAME), QStringLiteral("fileFromHome"));
+
+    // Compare what we get via kio_file and what we get when KFileItem stat()s directly
+    // for the requested fields
+    const KFileItem kioItem(entry, url);
+    const KFileItem fileItem(url);
+    QCOMPARE(kioItem.name(), fileItem.name());
+    QCOMPARE(kioItem.url(), fileItem.url());
+    QCOMPARE(kioItem.size(), fileItem.size());
+    QCOMPARE(kioItem.user(), "");
+    QCOMPARE(kioItem.group(), "");
+    QCOMPARE(kioItem.mimetype(), "application/octet-stream");
+    QCOMPARE(kioItem.permissions(), 438);
+    QCOMPARE(kioItem.time(KFileItem::ModificationTime), QDateTime());
+    QCOMPARE(kioItem.time(KFileItem::AccessTime), QDateTime());
+}
+
+void JobTest::statWithInode()
+{
+    const QString filePath = homeTmpDir() + "fileFromHome";
+    createTestFile(filePath);
+    const QUrl url(QUrl::fromLocalFile(filePath));
+    KIO::StatJob *job = KIO::statDetails(url, KIO::StatJob::SourceSide, KIO::StatInode);
+    QVERIFY(job);
+    QVERIFY2(job->exec(), qPrintable(job->errorString()));
+
+    const KIO::UDSEntry entry = job->statResult();
+    QVERIFY(entry.contains(KIO::UDSEntry::UDS_DEVICE_ID));
+    QVERIFY(entry.contains(KIO::UDSEntry::UDS_INODE));
+    QCOMPARE(entry.count(), 2);
+
+    const QString path = otherTmpDir() + "otherFile";
+    createTestFile(path);
+    const QUrl otherUrl(QUrl::fromLocalFile(path));
+    KIO::StatJob *otherJob = KIO::statDetails(otherUrl, KIO::StatJob::SourceSide, KIO::StatInode);
+    QVERIFY(otherJob);
+    QVERIFY2(otherJob->exec(), qPrintable(otherJob->errorString()));
+
+    const KIO::UDSEntry otherEntry = otherJob->statResult();
+    QVERIFY(otherEntry.contains(KIO::UDSEntry::UDS_DEVICE_ID));
+    QVERIFY(otherEntry.contains(KIO::UDSEntry::UDS_INODE));
+    QCOMPARE(otherEntry.count(), 2);
+
+    const int device = entry.numberValue(KIO::UDSEntry::UDS_DEVICE_ID);
+    const int otherDevice = otherEntry.numberValue(KIO::UDSEntry::UDS_DEVICE_ID);
+
+    // this test doesn't make sense on the CI as it's an LXC container with one partition
+    if (otherTmpDirIsOnSamePartition()) {
+        // On the CI where the two tmp dirs are on the only parition available
+        // in the LXC container, the device ID's would be identical
+        QCOMPARE(device, otherDevice);
+    }
+    else {
+        QVERIFY(device != otherDevice);
+    }
+}
+
 #ifndef Q_OS_WIN
 void JobTest::statSymlink()
 {
@@ -1424,13 +1588,30 @@ void JobTest::statSymlink()
     setTimeStamp(symlink, QDateTime::currentDateTime().addSecs(-20)); // differentiate link time and source file time
 
     const QUrl url(QUrl::fromLocalFile(symlink));
-    KIO::StatJob *job = KIO::stat(url, KIO::HideProgressInfo);
+    KIO::StatJob *job = KIO::statDetails(url, KIO::StatJob::StatSide::SourceSide,
+                                         KIO::StatBasic | KIO::StatResolveSymlink | KIO::StatUser | KIO::StatTime,
+                                         KIO::HideProgressInfo);
     QVERIFY(job);
     QVERIFY2(job->exec(), qPrintable(job->errorString()));
     // TODO set setSide, setDetails
     const KIO::UDSEntry &entry = job->statResult();
+
+    // we only get filename, access, type, size, linkdest, uid, gid, btime, mtime, atime
+    QVERIFY(entry.contains(KIO::UDSEntry::UDS_NAME));
+    QVERIFY(entry.contains(KIO::UDSEntry::UDS_ACCESS));
+    QVERIFY(entry.contains(KIO::UDSEntry::UDS_SIZE));
+    QVERIFY(entry.contains(KIO::UDSEntry::UDS_FILE_TYPE));
+    QVERIFY(entry.contains(KIO::UDSEntry::UDS_LINK_DEST));
+    QVERIFY(entry.contains(KIO::UDSEntry::UDS_USER));
+    QVERIFY(entry.contains(KIO::UDSEntry::UDS_GROUP));
+    //QVERIFY(entry.contains(KIO::UDSEntry::UDS_CREATION_TIME)); // only true if st_birthtime or statx is used
+    QVERIFY(entry.contains(KIO::UDSEntry::UDS_MODIFICATION_TIME));
+    QVERIFY(entry.contains(KIO::UDSEntry::UDS_ACCESS_TIME));
+    QCOMPARE(entry.count(), 9 + (entry.contains(KIO::UDSEntry::UDS_CREATION_TIME) ? 1 : 0));
+
     QVERIFY(!entry.isDir());
     QVERIFY(entry.isLink());
+    QVERIFY(entry.numberValue(KIO::UDSEntry::UDS_ACCESS) > 0);
     QCOMPARE(entry.stringValue(KIO::UDSEntry::UDS_NAME), QStringLiteral("link"));
 
     // Compare what we get via kio_file and what we get when KFileItem stat()s directly
@@ -1461,6 +1642,15 @@ void JobTest::mostLocalUrl()
     QCOMPARE(job->mostLocalUrl().toLocalFile(), filePath);
 }
 
+void JobTest::mostLocalUrlHttp() {
+    // the url is returned as-is, as an http url can't have a mostLocalUrl
+    const QUrl url("http://www.google.com");
+    KIO::StatJob *httpStat = KIO::mostLocalUrl(url, KIO::HideProgressInfo);
+    QVERIFY(httpStat);
+    QVERIFY2(httpStat->exec(), qPrintable(httpStat->errorString()));
+    QCOMPARE(httpStat->mostLocalUrl(), url);
+}
+
 void JobTest::chmodFile()
 {
     const QString filePath = homeTmpDir() + "fileForChmod";
@@ -1653,13 +1843,31 @@ void JobTest::moveDestAlreadyExistsAutoRename(const QString &destDir, bool moveD
     QVERIFY(QFile::exists(file3));
     QVERIFY(QFile::exists(file4));
 
-    QCOMPARE(spyRenamed.count(), 2);
+    QVERIFY(!spyRenamed.isEmpty());
+
     auto list = spyRenamed.takeFirst();
     QCOMPARE(list.at(1).toUrl(), QUrl::fromLocalFile(destDir + prefix + "(1)"));
     QCOMPARE(list.at(2).toUrl(), QUrl::fromLocalFile(file3));
-    list = spyRenamed.takeFirst();
-    QCOMPARE(list.at(1).toUrl(), QUrl::fromLocalFile(destDir + prefix + "(2)"));
-    QCOMPARE(list.at(2).toUrl(), QUrl::fromLocalFile(file4));
+
+    // Normally we'd see renamed(1, 3) and renamed(2, 4)
+    // But across partitions, direct rename fails, and we end up with a task list of
+    // 1->3, 2->3 since renaming 1 to 3 didn't happen yet.
+    // so renamed(2, 3) is emitted, as if the user had chosen that.
+    // And when that fails, we then get (3, 4)
+    if (spyRenamed.count() == 2) {
+        list = spyRenamed.takeFirst();
+        QCOMPARE(list.at(1).toUrl(), QUrl::fromLocalFile(destDir + prefix + "(2)"));
+        QCOMPARE(list.at(2).toUrl(), QUrl::fromLocalFile(file3));
+
+        list = spyRenamed.takeFirst();
+        QCOMPARE(list.at(1).toUrl(), QUrl::fromLocalFile(file3));
+        QCOMPARE(list.at(2).toUrl(), QUrl::fromLocalFile(file4));
+    } else {
+        QCOMPARE(spyRenamed.count(), 1);
+        list = spyRenamed.takeFirst();
+        QCOMPARE(list.at(1).toUrl(), QUrl::fromLocalFile(destDir + prefix + "(2)"));
+        QCOMPARE(list.at(2).toUrl(), QUrl::fromLocalFile(file4));
+    }
 
     if (moveDirs) {
         QDir().rmdir(file1);
@@ -1747,8 +1955,8 @@ void JobTest::safeOverwrite_data()
 {
     QTest::addColumn<bool>("destFileExists");
 
-    QTest::newRow("dest file exists") << true;
-    QTest::newRow("dest file doesn't exist") << false;
+    QTest::newRow("dest_file_exists") << true;
+    QTest::newRow("dest_file_does_not_exist") << false;
 }
 
 void JobTest::safeOverwrite()
@@ -1767,10 +1975,14 @@ void JobTest::safeOverwrite()
     createTestDirectory(srcDir);
     createTestDirectory(destDir);
 
-    QVERIFY(QFile::resize(srcFile, 1000000)); //~1MB
+    const int srcSize = 1000000; // ~1MB
+    QVERIFY(QFile::resize(srcFile, srcSize));
     if (!destFileExists) {
         QVERIFY(QFile::remove(destFile));
+    } else {
+        QVERIFY(QFile::exists(destFile));
     }
+    QVERIFY(!QFile::exists(destPartFile));
 
     if (otherTmpDirIsOnSamePartition()) {
         QSKIP(qPrintable(QStringLiteral("This test requires %1 and %2 to be on different partitions").arg(srcDir, destDir)));
@@ -1779,10 +1991,11 @@ void JobTest::safeOverwrite()
     KIO::FileCopyJob *job = KIO::file_move(QUrl::fromLocalFile(srcFile), QUrl::fromLocalFile(destFile), -1, KIO::HideProgressInfo | KIO::Overwrite);
     job->setUiDelegate(nullptr);
     QSignalSpy spyTotalSize(job, &KIO::FileCopyJob::totalSize);
-    connect(job, &KIO::FileCopyJob::totalSize, this, [destFileExists, destPartFile](KJob *job, qulonglong totalSize) {
+    connect(job, &KIO::FileCopyJob::processedSize, this, [&](KJob *job, qulonglong size) {
         Q_UNUSED(job);
-        if (totalSize > 0) {
-            QCOMPARE(destFileExists, QFile::exists(destPartFile));
+        if (size > 0 && size < srcSize) {
+            // To avoid overwriting dest, we want the kioslave to use dest.part
+            QCOMPARE(QFileInfo::exists(destPartFile), destFileExists);
         }
     });
     QVERIFY2(job->exec(), qPrintable(job->errorString()));
