@@ -38,17 +38,19 @@ public:
     QDateTime mBirthDate;
     bool mOnHold;
     QUrl mUrl;
+    bool mHasTempAuth;
 };
 
 IdleSlave::IdleSlave(QObject *parent)
     : QObject(parent), d(new IdleSlavePrivate)
 {
-    QObject::connect(&d->mConn, SIGNAL(readyRead()), this, SLOT(gotInput()));
+    QObject::connect(&d->mConn, &Connection::readyRead, this, &IdleSlave::gotInput);
     // Send it a SLAVE_STATUS command.
     d->mConn.send(CMD_SLAVE_STATUS);
     d->mPid = 0;
     d->mBirthDate = QDateTime::currentDateTime();
     d->mOnHold = false;
+    d->mHasTempAuth = false;
 }
 
 IdleSlave::~IdleSlave()
@@ -65,7 +67,11 @@ void IdleSlave::gotInput()
         deleteLater();
     } else if (cmd == MSG_SLAVE_ACK) {
         deleteLater();
-    } else if (cmd != MSG_SLAVE_STATUS) {
+    } else if (cmd != MSG_SLAVE_STATUS_V2
+#if KIOCORE_BUILD_DEPRECATED_SINCE(5, 45)
+        && cmd != MSG_SLAVE_STATUS
+#endif
+                                  ) {
         qCritical() << "Unexpected data from KIO slave.";
         deleteLater();
     } else {
@@ -74,15 +80,26 @@ void IdleSlave::gotInput()
         QByteArray protocol;
         QString host;
         qint8 b;
-        stream >> pid >> protocol >> host >> b;
         // Overload with (bool) onHold, (QUrl) url.
-        if (!stream.atEnd()) {
+        stream >> pid >> protocol >> host >> b;
+        if (cmd == MSG_SLAVE_STATUS_V2) {
             QUrl url;
-            stream >> url;
-            d->mOnHold = true;
-            d->mUrl = url;
+            bool onHold;
+            bool tempAuth;
+            stream >> onHold >> url >> tempAuth;
+            d->mHasTempAuth = tempAuth;
+            if (onHold) {
+                d->mOnHold = onHold;
+                d->mUrl = url;
+            }
+        } else { // compat code for KF < 5.45. TODO KF6: remove
+            if (!stream.atEnd()) {
+                QUrl url;
+                stream >> url;
+                d->mOnHold = true;
+                d->mUrl = url;
+            }
         }
-
         d->mPid = pid;
         d->mConnected = (b != 0);
         d->mProtocol = QString::fromLatin1(protocol);
@@ -143,3 +160,8 @@ Connection *IdleSlave::connection() const
 {
     return &d->mConn;
 }
+
+bool IdleSlave::hasTempAuthorization() const
+{
+    return d->mHasTempAuth;
+}
