@@ -21,7 +21,7 @@
 #ifndef KPROTOCOLMANAGER_H
 #define KPROTOCOLMANAGER_H
 
-#include <QtCore/QStringList>
+#include <QStringList>
 
 #include "kio/global.h" // KIO::CacheControl
 #include "kiocore_export.h"
@@ -37,6 +37,8 @@ class SlaveConfigPrivate;
 } // namespace KIO
 
 /**
+ * @class KProtocolManager kprotocolmanager.h <KProtocolManager>
+ *
  * Provides information about I/O (Internet, etc.) settings chosen/set
  * by the end user.
  *
@@ -114,7 +116,7 @@ public:
      */
     static QString userAgentForHost(const QString &hostname);
 
-    /*
+    /**
      * Returns system name, version and machine type, for example "Windows", "5.1", "i686".
      * This information can be used for constructing custom user-agent strings.
      *
@@ -465,6 +467,18 @@ public:
      */
     static bool supportsOpening(const QUrl &url);
 
+    /**
+     * Returns whether the protocol can be truncated with FileJob::truncate(KIO::filesize_t length).
+     *
+     * This corresponds to the "truncating=" field in the protocol description file.
+     * Valid values for this field are "true" or "false" (default).
+     *
+     * @param url the url to check
+     * @return true if the protocol supports truncating
+     * @since 5.66
+     */
+    static bool supportsTruncating(const QUrl &url);
+
     /**
      * Returns whether the protocol can copy files/objects directly from the
      * filesystem itself. If not, the application will read files from the
