@@ -27,7 +27,7 @@
 
 #include <QUrl>
 #include <QWidget>
-#include <QtCore/QByteArray>
+#include <QByteArray>
 
 class KFilePlacesModel;
 class KUrlComboBox;
@@ -270,8 +270,7 @@ public:
      * If an application supports only some special protocols, they can be set
      * with \a protocols .
      */
-    // KDE5: Think about removing the custom-protocols-property. It had been used
-    // only by one application currently which uses a different approach now.
+    // TODO KF6 rename to setSupportedSchemes to match KDirOperator and KFileWidget
     void setCustomProtocols(const QStringList &protocols);
 
     /**
@@ -279,13 +278,27 @@ public:
      */
     QStringList customProtocols() const;
 
-#if !defined(KIOFILEWIDGETS_NO_DEPRECATED) && !defined(DOXYGEN_SHOULD_SKIP_THIS)
+    /**
+     * The child widget that received the QDropEvent when dropping on the URL
+     * navigator. You can pass this widget to KJobWidgets::setWindow()
+     * if you need to show a drop menu with KIO::drop().
+     * @return Child widget that has received the last drop event, or nullptr if
+     *         nothing has been dropped yet on the URL navigator.
+     * @since 5.37
+     * @see KIO::drop()
+     */
+    QWidget *dropWidget() const;
+
+#if KIOFILEWIDGETS_ENABLE_DEPRECATED_SINCE(4, 5)
     /**
      * @return     The current URL of the location.
-     * @deprecated Use KUrlNavigator::locationUrl() instead.
+     * @deprecated Since 4.5, use KUrlNavigator::locationUrl() instead.
      */
-    KIOFILEWIDGETS_DEPRECATED const QUrl &url() const;
+    KIOFILEWIDGETS_DEPRECATED_VERSION(4, 5, "Use KUrlNavigator::locationUrl(int)")
+    const QUrl &url() const;
+#endif
 
+#if KIOFILEWIDGETS_ENABLE_DEPRECATED_SINCE(4, 5)
     /**
      * @return The portion of the current URL up to the path part given
      * by \a index. Assuming that the current URL is /home/peter/Documents/Music,
@@ -294,33 +307,46 @@ public:
      * - index is 1: /home/peter
      * - index is 2: /home/peter/Documents
      * - index >= 3: /home/peter/Documents/Music
-     * @deprecated It should not be necessary for a client of KUrlNavigator to query this information.
+     * @deprecated Since 4.5. It should not be necessary for a client of KUrlNavigator to query this information.
      */
-    KIOFILEWIDGETS_DEPRECATED QUrl url(int index) const;
+    KIOFILEWIDGETS_DEPRECATED_VERSION(4, 5, "Do not use")
+    QUrl url(int index) const;
+#endif
 
+#if KIOFILEWIDGETS_ENABLE_DEPRECATED_SINCE(4, 5)
     /**
      * @return URL for the history element with the index \a historyIndex.
      *         The history index 0 represents the most recent URL.
      * @since 4.3
-     * @deprecated Use KUrlNavigator::locationUrl(historyIndex) instead.
+     * @deprecated Since 4.5, use KUrlNavigator::locationUrl(historyIndex) instead.
      */
-    KIOFILEWIDGETS_DEPRECATED QUrl historyUrl(int historyIndex) const;
+    KIOFILEWIDGETS_DEPRECATED_VERSION(4, 5, "Use KUrlNavigator::locationUrl(int)")
+    QUrl historyUrl(int historyIndex) const;
+#endif
 
+#if KIOFILEWIDGETS_ENABLE_DEPRECATED_SINCE(4, 5)
     /**
      * @return The saved root URL for the current URL (see KUrlNavigator::saveRootUrl()).
-     * @deprecated Use KUrlNavigator::locationState() instead.
+     * @deprecated Since 4.5, use KUrlNavigator::locationState() instead.
      */
-    KIOFILEWIDGETS_DEPRECATED const QUrl &savedRootUrl() const;
+    KIOFILEWIDGETS_DEPRECATED_VERSION(4, 5, "Use KUrlNavigator::locationState(int)")
+    const QUrl &savedRootUrl() const;
+#endif
 
+#if KIOFILEWIDGETS_ENABLE_DEPRECATED_SINCE(4, 5)
     /**
      * @return The saved contents position of the upper left corner
      *         for the current URL.
-     * @deprecated Use KUrlNavigator::locationState() instead.
+     * @deprecated Since 4.5, use KUrlNavigator::locationState() instead.
      */
-    KIOFILEWIDGETS_DEPRECATED QPoint savedPosition() const;
+    KIOFILEWIDGETS_DEPRECATED_VERSION(4, 5, "Use KUrlNavigator::locationState(int)")
+    QPoint savedPosition() const;
+#endif
 
-    /** @deprecated Use setHomeUrl(const QUrl& url) instead. */
-    KIOFILEWIDGETS_DEPRECATED void setHomeUrl(const QString &homeUrl);
+#if KIOFILEWIDGETS_ENABLE_DEPRECATED_SINCE(4, 5)
+    /** @deprecated Since 4.5, use setHomeUrl(const QUrl& url) instead. */
+    KIOFILEWIDGETS_DEPRECATED_VERSION(4, 5, "Use KUrlNavigator::setHomeUrl(const QUrl&)")
+    void setHomeUrl(const QString &homeUrl);
 #endif
 
 public Q_SLOTS:
@@ -340,29 +366,36 @@ public Q_SLOTS:
      */
     void requestActivation();
 
-#if !defined(DOXYGEN_SHOULD_SKIP_THIS)
+#if !defined(K_DOXYGEN)
     // KDE5: Remove and listen for focus-signal instead
     void setFocus();
 #endif
 
-#if !defined(KIOFILEWIDGETS_NO_DEPRECATED) && !defined(DOXYGEN_SHOULD_SKIP_THIS)
+#if KIOFILEWIDGETS_ENABLE_DEPRECATED_SINCE(4, 5)
     /**
      * Sets the location to \a url.
-     * @deprecated Use KUrlNavigator::setLocationUrl(url).
+     * @deprecated Since 4.5, use KUrlNavigator::setLocationUrl(url).
      */
-    KIOFILEWIDGETS_DEPRECATED void setUrl(const QUrl &url);
+    KIOFILEWIDGETS_DEPRECATED_VERSION(4, 5, "Use KUrlNavigator::setLocationUrl(const QUrl&))")
+    void setUrl(const QUrl &url);
+#endif
 
+#if KIOFILEWIDGETS_ENABLE_DEPRECATED_SINCE(4, 5)
     /**
      * Saves the used root URL of the content for the current history element.
-     * @deprecated Use KUrlNavigator::saveLocationState() instead.
+     * @deprecated Since 4.5, use KUrlNavigator::saveLocationState() instead.
      */
-    KIOFILEWIDGETS_DEPRECATED void saveRootUrl(const QUrl &url);
+    KIOFILEWIDGETS_DEPRECATED_VERSION(4, 5, "Use KUrlNavigator::saveLocationState(const QByteArray &)")
+    void saveRootUrl(const QUrl &url);
+#endif
 
+#if KIOFILEWIDGETS_ENABLE_DEPRECATED_SINCE(4, 5)
     /**
      * Saves the coordinates of the contents for the current history element.
-     * @deprecated Use KUrlNavigator::saveLocationState() instead.
+     * @deprecated Since 4.5, use KUrlNavigator::saveLocationState() instead.
      */
-    KIOFILEWIDGETS_DEPRECATED void savePosition(int x, int y);
+    KIOFILEWIDGETS_DEPRECATED_VERSION(4, 5, "Use KUrlNavigator::saveLocationState(const QByteArray &)")
+    void savePosition(int x, int y);
 #endif
 
 Q_SIGNALS:
@@ -421,32 +454,49 @@ Q_SIGNALS:
      */
     void tabRequested(const QUrl &url);
 
+    /**
+     * When the URL is changed and the new URL (e.g. /home/user1/)
+     * is a parent of the previous URL (e.g. /home/user1/data/stuff),
+     * then this signal is emitted and \a url is set to the child
+     * directory of the new URL which is an ancestor of the old URL
+     * (in the example paths this would be /home/user1/data/).
+     * This signal allows file managers to pre-select the directory
+     * that the user is navigating up from.
+     * @since 5.37.0
+     */
+    void urlSelectionRequested(const QUrl &url);
+
 protected:
-#if !defined(DOXYGEN_SHOULD_SKIP_THIS)
+#if !defined(K_DOXYGEN)
     /**
      * If the Escape key is pressed, the navigation bar should switch
      * to the breadcrumb view.
      * @see QWidget::keyPressEvent()
      */
-    void keyPressEvent(QKeyEvent *event) Q_DECL_OVERRIDE;
+    void keyPressEvent(QKeyEvent *event) override;
 
     /**
      * Reimplemented for internal purposes.
      */
-    void keyReleaseEvent(QKeyEvent *event) Q_DECL_OVERRIDE;
+    void keyReleaseEvent(QKeyEvent *event) override;
 
     /**
      * Paste the clipboard content as URL, if the middle mouse
      * button has been clicked.
      * @see QWidget::mouseReleaseEvent()
      */
-    void mouseReleaseEvent(QMouseEvent *event) Q_DECL_OVERRIDE;
+    void mouseReleaseEvent(QMouseEvent *event) override;
 
-    void resizeEvent(QResizeEvent *event) Q_DECL_OVERRIDE;
+    /**
+     * Reimplemented to activate on middle mousse button click
+     */
+    void mousePressEvent(QMouseEvent *event) override;
 
-    void wheelEvent(QWheelEvent *event) Q_DECL_OVERRIDE;
+    void resizeEvent(QResizeEvent *event) override;
 
-    bool eventFilter(QObject *watched, QEvent *event) Q_DECL_OVERRIDE;
+    void wheelEvent(QWheelEvent *event) override;
+
+    bool eventFilter(QObject *watched, QEvent *event) override;
 #endif
 
 private:
@@ -454,8 +504,8 @@ private:
     Q_PRIVATE_SLOT(d, void slotProtocolChanged(const QString &protocol))
     Q_PRIVATE_SLOT(d, void slotToggleEditableButtonPressed())
     Q_PRIVATE_SLOT(d, void dropUrls(const QUrl &destination, QDropEvent *))
-    Q_PRIVATE_SLOT(d, void slotNavigatorButtonClicked(const QUrl &url, Qt::MouseButton button))
-    Q_PRIVATE_SLOT(d, void openContextMenu())
+    Q_PRIVATE_SLOT(d, void slotNavigatorButtonClicked(const QUrl &url, Qt::MouseButton button, Qt::KeyboardModifiers modifiers))
+    Q_PRIVATE_SLOT(d, void openContextMenu(QPoint))
     Q_PRIVATE_SLOT(d, void openPathSelectorMenu())
     Q_PRIVATE_SLOT(d, void updateButtonVisibility())
     Q_PRIVATE_SLOT(d, void switchToBreadcrumbMode())
