@@ -20,7 +20,6 @@
 
 #include "kfileitemactions.h"
 #include "kfileitemactions_p.h"
-#include <krun.h>
 #include <kmimetypetrader.h>
 #include <kdesktopfileactions.h>
 #include <klocalizedstring.h>
@@ -30,6 +29,8 @@
 #include <kservicetypetrader.h>
 #include <KAbstractFileItemActionPlugin>
 #include <KPluginMetaData>
+#include <KIO/ApplicationLauncherJob>
+#include <KIO/JobUiDelegate>
 
 #include <QFile>
 #include <QMenu>
@@ -254,8 +255,7 @@ int KFileItemActions::addServiceActionsTo(QMenu *mainMenu)
     KIO::PopupServices s;
 
     // 1 - Look for builtin and user-defined services
-    if (isSingleLocal && (d->m_props.mimeType() == QLatin1String("application/x-desktop") || // .desktop file
-                          d->m_props.mimeType() == QLatin1String("inode/blockdevice"))) { // dev file
+    if (isSingleLocal && d->m_props.mimeType() == QLatin1String("application/x-desktop")) {
         // get builtin services, like mount/unmount
         const QString path = firstItem.localPath();
         s.builtin = KDesktopFileActions::builtinServices(QUrl::fromLocalFile(path));
@@ -724,12 +724,11 @@ void KFileItemActionsPrivate::slotRunPreferredApplications()
             continue;
         }
 
-        const KService::Ptr servicePtr = KService::serviceByStorageId(serviceId);
-        if (!servicePtr) {
-            KRun::displayOpenWithDialog(serviceItems.urlList(), m_parentWidget);
-            continue;
-        }
-        KRun::runApplication(*servicePtr, serviceItems.urlList(), m_parentWidget);
+        const KService::Ptr servicePtr = KService::serviceByStorageId(serviceId); // can be nullptr
+        KIO::ApplicationLauncherJob *job = new KIO::ApplicationLauncherJob(servicePtr);
+        job->setUrls(serviceItems.urlList());
+        job->setUiDelegate(new KIO::JobUiDelegate(KJobUiDelegate::AutoHandlingEnabled, m_parentWidget));
+        job->start();
     }
 }
 
@@ -750,17 +749,24 @@ void KFileItemActionsPrivate::openWithByMime(const KFileItemList &fileItems)
                 mimeItems << item;
             }
         }
-        KRun::displayOpenWithDialog(mimeItems.urlList(), m_parentWidget);
+        // Show Open With dialog
+        KIO::ApplicationLauncherJob *job = new KIO::ApplicationLauncherJob();
+        job->setUrls(mimeItems.urlList());
+        job->setUiDelegate(new KIO::JobUiDelegate(KJobUiDelegate::AutoHandlingEnabled, m_parentWidget));
+        job->start();
     }
 }
 
 void KFileItemActionsPrivate::slotRunApplication(QAction *act)
 {
-    // Is it an application, from one of the "Open With" actions
+    // Is it an application, from one of the "Open With" actions?
     KService::Ptr app = act->data().value<KService::Ptr>();
     Q_ASSERT(app);
     if (app) {
-        KRun::runApplication(*app, m_props.urlList(), m_parentWidget);
+        KIO::ApplicationLauncherJob *job = new KIO::ApplicationLauncherJob(app);
+        job->setUrls(m_props.urlList());
+        job->setUiDelegate(new KIO::JobUiDelegate(KJobUiDelegate::AutoHandlingEnabled, m_parentWidget));
+        job->start();
     }
 }
 
@@ -768,7 +774,10 @@ void KFileItemActionsPrivate::slotOpenWithDialog()
 {
     // The item 'Other...' or 'Open With...' has been selected
     emit q->openWithDialogAboutToBeShown();
-    KRun::displayOpenWithDialog(m_props.urlList(), m_parentWidget);
+    KIO::ApplicationLauncherJob *job = new KIO::ApplicationLauncherJob();
+    job->setUrls(m_props.urlList());
+    job->setUiDelegate(new KIO::JobUiDelegate(KJobUiDelegate::AutoHandlingEnabled, m_parentWidget));
+    job->start();
 }
 
 QStringList KFileItemActionsPrivate::listMimeTypes(const KFileItemList &items)
