@@ -40,7 +40,7 @@
 #include <QImage>
 #include <QPixmap>
 #include <QTimer>
-#include <QRegExp>
+#include <QRegularExpression>
 #include <qtemporaryfile.h>
 #include <qsavefile.h>
 
@@ -50,7 +50,6 @@
 #include <kservicetypetrader.h>
 #include <kservice.h>
 #include <ksharedconfig.h>
-#include <QLinkedList>
 #include <kconfiggroup.h>
 #include <kprotocolinfo.h>
 #include <qmimedatabase.h>
@@ -107,8 +106,8 @@ public:
     // Some plugins support remote URLs, <protocol, mimetypes>
     QHash<QString, QStringList> m_remoteProtocolPlugins;
     // Our todo list :)
-    // We remove the first item at every step, so use QLinkedList
-    QLinkedList<PreviewItem> items;
+    // We remove the first item at every step, so use std::list
+    std::list<PreviewItem> items;
     // The current item
     PreviewItem currentItem;
     // The modification time of that URL
@@ -148,6 +147,8 @@ public:
     uchar *shmaddr;
     // Root of thumbnail cache
     QString thumbRoot;
+    // List of encrypted mount points for checking if we should save thumbnail
+    KMountPoint::List encryptedMountsList;
 
     void getOrCreateThumbnail();
     bool statResultThumbnail();
@@ -162,7 +163,7 @@ public:
     Q_DECLARE_PUBLIC(PreviewJob)
 };
 
-#ifndef KIOWIDGETS_NO_DEPRECATED
+#if KIOWIDGETS_BUILD_DEPRECATED_SINCE(4, 7)
 PreviewJob::PreviewJob(const KFileItemList &items, int width, int height,
                        int iconSize, int iconAlpha, bool scale, bool save,
                        const QStringList *enabledPlugins)
@@ -280,12 +281,12 @@ void PreviewJobPrivate::startPreview()
         if (!p.isEmpty()) {
             protocols.append(p);
         }
-        foreach (const QString &protocol, protocols) {
+        for (const QString &protocol : qAsConst(protocols)) {
             // We cannot use mimeTypes() here, it doesn't support groups such as: text/*
             const QStringList mtypes = (*it)->serviceTypes();
             // Add supported mimetype for this protocol
             QStringList &_ms = m_remoteProtocolPlugins[protocol];
-            foreach (const QString &_m, mtypes) {
+            for (const QString &_m : mtypes) {
                 if (_m != QLatin1String("ThumbCreator")) {
                     protocolMap[protocol].insert(_m, *it);
                     if (!_ms.contains(_m)) {
@@ -304,10 +305,9 @@ void PreviewJobPrivate::startPreview()
         }
     }
 
+    //Prepare encryptedMountsList which will be used in ::slotThumbData
     const auto mountsList = KMountPoint::currentMountPoints();
-    KMountPoint::List encryptedMountsList;
     const auto thumbRootMount = mountsList.findByPath(thumbRoot);
-
     std::copy_if(mountsList.begin(), mountsList.end(),
                  std::back_inserter(encryptedMountsList),
                  [&thumbRootMount] (KMountPoint::Ptr mount) {
@@ -324,10 +324,6 @@ void PreviewJobPrivate::startPreview()
         PreviewItem item;
         item.item = *kit;
 
-        if (encryptedMountsList.findByPath(item.item.localPath())) {
-            continue;
-        }
-
         const QString mimeType = item.item.mimetype();
         KService::Ptr plugin(nullptr);
 
@@ -341,7 +337,7 @@ void PreviewJobPrivate::startPreview()
             QMap<QString, KService::Ptr>::ConstIterator pluginIt = mimeMap.constFind(mimeType);
             if (pluginIt == mimeMap.constEnd()) {
                 QString groupMimeType = mimeType;
-                groupMimeType.replace(QRegExp(QStringLiteral("/.*")), QStringLiteral("/*"));
+                groupMimeType.replace(QRegularExpression(QStringLiteral("/.*")), QStringLiteral("/*"));
                 pluginIt = mimeMap.constFind(groupMimeType);
 
                 if (pluginIt == mimeMap.constEnd()) {
@@ -350,7 +346,7 @@ void PreviewJobPrivate::startPreview()
                     const QMimeType mimeInfo = db.mimeTypeForName(mimeType);
                     if (mimeInfo.isValid()) {
                         const QStringList parentMimeTypes = mimeInfo.allAncestors();
-                        Q_FOREACH (const QString &parentMimeType, parentMimeTypes) {
+                        for (const QString &parentMimeType : parentMimeTypes) {
                             pluginIt = mimeMap.constFind(parentMimeType);
                             if (pluginIt != mimeMap.constEnd()) {
                                 break;
@@ -367,7 +363,7 @@ void PreviewJobPrivate::startPreview()
 
         if (plugin) {
             item.plugin = plugin;
-            items.append(item);
+            items.push_back(item);
             if (!bNeedCache && bSave && plugin->property(QStringLiteral("CacheThumbnail")).toBool()) {
                 const QUrl url = (*kit).url();
                 if (!url.isLocalFile() ||
@@ -408,11 +404,15 @@ void PreviewJobPrivate::startPreview()
 void PreviewJob::removeItem(const QUrl &url)
 {
     Q_D(PreviewJob);
-    for (QLinkedList<PreviewItem>::Iterator it = d->items.begin(); it != d->items.end(); ++it)
+
+    auto it = d->items.cbegin();
+    while (it != d->items.cend()) {
         if ((*it).item.url() == url) {
             d->items.erase(it);
             break;
         }
+        ++it;
+    }
 
     if (d->currentItem.item.url() == url) {
         KJob *job = subjobs().first();
@@ -455,15 +455,15 @@ void PreviewJobPrivate::determineNextFile()
         }
     }
     // No more items ?
-    if (items.isEmpty()) {
+    if (items.empty()) {
         q->emitResult();
         return;
     } else {
         // First, stat the orig file
         state = PreviewJobPrivate::STATE_STATORIG;
-        currentItem = items.first();
+        currentItem = items.front();
+        items.pop_front();
         succeeded = false;
-        items.removeFirst();
         KIO::Job *job = KIO::stat(currentItem.item.url(), KIO::HideProgressInfo);
         job->addMetaData(QStringLiteral("thumbnail"), QStringLiteral("1"));
         job->addMetaData(QStringLiteral("no-auth-prompt"), QStringLiteral("true"));
@@ -572,7 +572,7 @@ bool PreviewJobPrivate::statResultThumbnail()
 
     QString thumbnailerVersion = currentItem.plugin->property(QStringLiteral("ThumbnailerVersion"), QVariant::String).toString();
 
-    if (!thumbnailerVersion.isEmpty() && thumb.text(QStringLiteral("Software")).startsWith(QStringLiteral("KDE Thumbnail Generator"))) {
+    if (!thumbnailerVersion.isEmpty() && thumb.text(QStringLiteral("Software")).startsWith(QLatin1String("KDE Thumbnail Generator"))) {
         //Check if the version matches
         //The software string should read "KDE Thumbnail Generator pluginName (vX)"
         QString softwareString = thumb.text(QStringLiteral("Software")).remove(QStringLiteral("KDE Thumbnail Generator")).trimmed();
@@ -694,11 +694,12 @@ void PreviewJobPrivate::createThumbnail(const QString &pixPath)
 
 void PreviewJobPrivate::slotThumbData(KIO::Job *, const QByteArray &data)
 {
+    const bool isEncrypted = encryptedMountsList.findByPath(currentItem.item.url().toLocalFile());
     bool save = bSave &&
+                !sequenceIndex && !isEncrypted &&
                 currentItem.plugin->property(QStringLiteral("CacheThumbnail")).toBool() &&
                 (!currentItem.item.url().isLocalFile() ||
-                 !currentItem.item.url().adjusted(QUrl::RemoveFilename).toLocalFile().startsWith(thumbRoot))
-                && !sequenceIndex;
+                 !currentItem.item.url().adjusted(QUrl::RemoveFilename).toLocalFile().startsWith(thumbRoot));
     QImage thumb;
 #if WITH_SHM
     if (shmaddr) {
@@ -756,20 +757,22 @@ QStringList PreviewJob::availablePlugins()
 {
     QStringList result;
     const KService::List plugins = KServiceTypeTrader::self()->query(QStringLiteral("ThumbCreator"));
-    for (KService::List::ConstIterator it = plugins.begin(); it != plugins.end(); ++it)
-        if (!result.contains((*it)->desktopEntryName())) {
-            result.append((*it)->desktopEntryName());
+    for (const KService::Ptr &plugin : plugins) {
+        const QString desktopEntryName = plugin->desktopEntryName();
+        if (!result.contains(desktopEntryName)) {
+            result.append(desktopEntryName);
         }
+    }
     return result;
 }
 
 QStringList PreviewJob::defaultPlugins()
 {
-    QStringList blacklist = QStringList()
+    const QStringList blacklist = QStringList()
                             << QStringLiteral("textthumbnail");
 
     QStringList defaultPlugins = availablePlugins();
-    foreach (const QString plugin, blacklist) {
+    for (const QString &plugin : blacklist) {
         defaultPlugins.removeAll(plugin);
     }
 
@@ -780,13 +783,13 @@ QStringList PreviewJob::supportedMimeTypes()
 {
     QStringList result;
     const KService::List plugins = KServiceTypeTrader::self()->query(QStringLiteral("ThumbCreator"));
-    for (KService::List::ConstIterator it = plugins.begin(); it != plugins.end(); ++it) {
-        result += (*it)->mimeTypes();
+    for (const KService::Ptr &plugin : plugins) {
+        result += plugin->mimeTypes();
     }
     return result;
 }
 
-#ifndef KIOWIDGETS_NO_DEPRECATED
+#if KIOWIDGETS_BUILD_DEPRECATED_SINCE(4, 7)
 PreviewJob *KIO::filePreview(const KFileItemList &items, int width, int height,
                              int iconSize, int iconAlpha, bool scale, bool save,
                              const QStringList *enabledPlugins)
@@ -794,16 +797,18 @@ PreviewJob *KIO::filePreview(const KFileItemList &items, int width, int height,
     return new PreviewJob(items, width, height, iconSize, iconAlpha,
                           scale, save, enabledPlugins);
 }
+#endif
 
+#if KIOWIDGETS_BUILD_DEPRECATED_SINCE(4, 7)
 PreviewJob *KIO::filePreview(const QList<QUrl> &items, int width, int height,
                              int iconSize, int iconAlpha, bool scale, bool save,
                              const QStringList *enabledPlugins)
 {
     KFileItemList fileItems;
     fileItems.reserve(items.size());
-    for (QList<QUrl>::const_iterator it = items.begin(); it != items.end(); ++it) {
-        Q_ASSERT((*it).isValid());   // please call us with valid urls only
-        fileItems.append(KFileItem(*it));
+    for (const QUrl &url : items) {
+        Q_ASSERT(url.isValid());   // please call us with valid urls only
+        fileItems.append(KFileItem(url));
     }
     return new PreviewJob(fileItems, width, height, iconSize, iconAlpha,
                           scale, save, enabledPlugins);
@@ -815,7 +820,7 @@ PreviewJob *KIO::filePreview(const KFileItemList &items, const QSize &size, cons
     return new PreviewJob(items, size, enabledPlugins);
 }
 
-#ifndef KIOWIDGETS_NO_DEPRECATED
+#if KIOWIDGETS_BUILD_DEPRECATED_SINCE(4, 5)
 KIO::filesize_t PreviewJob::maximumFileSize()
 {
     KConfigGroup cg(KSharedConfig::openConfig(), "PreviewSettings");
